#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2007 Insecure.Com LLC.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

# This file contains global definitions of program names. The plain names are
# the usually lower-case program or package names. The display names are
# properly capitalized for use in human-readable sentences.

APP_NAME = "zenmap"
APP_DISPLAY_NAME = "Zenmap"
APP_WEB_SITE = "http://nmap.org/zenmap"
APP_DOWNLOAD_SITE = "http://nmap.org/download.html"
APP_COPYRIGHT = "Copyright 2005-2008 Insecure.Com LLC"

NMAP_DISPLAY_NAME = u"Nmap"
NMAP_WEB_SITE = "http://nmap.org"

UMIT_DISPLAY_NAME = "Umit"
UMIT_WEB_SITE = "http://www.umitproject.org/"
